"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bodyParser = require("body-parser");
const express = require("express");
const session = require("express-session");
const either = require("fp-ts/lib/Either");
const http = require("http");
const t = require("io-ts");
const other_1 = require("./helpers/other");
const index_1 = require("./index");
const app = express();
app.use(session({ secret: 'foo' }));
// We parse JSON using io-ts.
app.use(bodyParser.text({ type: 'application/json' }));
const requestTypes = index_1.createValidatedRequestTypes({
    session: t.interface({}),
    body: other_1.composeTypes(other_1.JSONFromString, t.interface({
        name: t.string,
    }), 'Body'),
    query: t.interface({
        age: other_1.NumberFromString,
    }),
});
const requestHandler = index_1.wrapValidatedRequestHandler({
    // These values will be validated against their types. These also serve as static types for
    // compile time type checking and auto completion.
    types: requestTypes,
    // If there is an error when validating the request, this maps it to our error type.
    createValidationErrorsError: other_1.formatValidationErrors,
    // After validation, we can use the validated request (session, body, query) to compute our
    // response.
    handler: (validatedReq) => 
    // Here the type checker knows the type of our request’s session, body, and query objects.
    // E.g. `body.name` is type `string` and `query.age` is type `number`.
    either.right({ result: `name: ${validatedReq.body.name}, age: ${validatedReq.query.age}` }),
    // Once we've computed our response, we can inform Express so it can respond to the request.
    // In the future these could be incorporated into the return type of the `handler` function.
    errorResponseHandler: (_req, res) => (errorResponse) => {
        res.status(500).send(errorResponse);
    },
    successResponseHandler: (_req, res) => (successResponse) => {
        res.status(200).send(successResponse);
    },
});
app.post('/', requestHandler);
// app.post('/', (req, res) => {
//     console.log(req.body);
//     res.send(200)
// });
const onListen = (server) => {
    const { port } = server.address();
    console.log(`Server running on port ${port}`);
};
const httpServer = http.createServer(app);
httpServer.listen(8080, () => {
    onListen(httpServer);
});
// ❯ curl --request POST --silent "localhost:8080/" | jq '.'
// [
//   "Expecting NumberFromString at query.age but instead got: undefined.",
//   "Expecting Body at body but instead got: {}."
// ]
// ❯ curl --request POST --silent --header 'Content-Type: application/json' \
//     --data '{ "name": "bob" }' "localhost:8080/?age=foo" | jq '.'
// [
//   "Expecting NumberFromString at query.age but instead got: \"foo\"."
// ]
// ❯ curl --request POST --silent --header 'Content-Type: application/json' \
//     --data '{ "name": "bob" }' "localhost:8080/?age=5" | jq '.'
// {
//   "result": "name: bob, age: 5"
// }
//# sourceMappingURL=example.js.map